@@ -25,9 +25,7 @@
 #include <glib-object.h>
 
 #include <gtk/gtk.h>
-#include <nm-client.h>
-#include <nm-access-point.h>
-#include <nm-remote-settings.h>
+#include <NetworkManager.h>
 
 G_BEGIN_DECLS
 
@@ -43,12 +41,11 @@ typedef struct _NetConnectionEditorClass     NetConnectionEditorClass;
 
 struct _NetConnectionEditor
 {
-         GObject parent;
+        GtkDialog parent;
 
         GtkWidget        *parent_window;
         NMClient         *client;
         NMDevice         *device;
-        NMRemoteSettings *settings;
 
         NMConnection     *connection;
         NMConnection     *orig_connection;
@@ -56,19 +53,25 @@ struct _NetConnectionEditor
         gboolean          is_changed;
         NMAccessPoint    *ap;
 
-        GtkBuilder       *builder;
-        GtkWidget        *window;
+        GtkButton        *cancel_button;
+        GtkButton        *apply_button;
+        GtkNotebook      *notebook;
+        GtkFrame         *add_connection_frame;
+        GtkNotebook      *toplevel_notebook;
 
         GSList *initializing_pages;
         GSList *pages;
 
         guint                    permission_id;
         NMClientPermissionResult can_modify;
+
+        gboolean          title_set;
+        gboolean          show_when_initialized;
 };
 
 struct _NetConnectionEditorClass
 {
-        GObjectClass parent_class;
+        GtkDialogClass parent_class;
 
         void (*done) (NetConnectionEditor *details, gboolean success);
 };
@@ -78,10 +81,10 @@ NetConnectionEditor *net_connection_editor_new      (GtkWindow        *parent_wi
                                                      NMConnection     *connection,
                                                      NMDevice         *device,
                                                      NMAccessPoint    *ap,
-                                                     NMClient         *client,
-                                                     NMRemoteSettings *settings);
+                                                     NMClient         *client);
+void                 net_connection_editor_set_title (NetConnectionEditor  *editor,
+                                                      const gchar          *title);
 void                 net_connection_editor_run      (NetConnectionEditor   *editor);
-void                 net_connection_editor_present  (NetConnectionEditor   *editor);
 void                 net_connection_editor_forget   (NetConnectionEditor   *editor);
 void                 net_connection_editor_reset    (NetConnectionEditor   *editor);
 
