@@ -157,6 +157,7 @@ add_provider_row (CcGoaPanel  *self,
     }
 
   image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_DIALOG);
+  gtk_style_context_add_class (gtk_widget_get_style_context (image), "lowres-icon");
   gtk_widget_show (image);
   gtk_container_add (GTK_CONTAINER (row_grid), image);
   g_object_set (image, "margin", 6, NULL);
@@ -612,6 +613,11 @@ show_page_account (CcGoaPanel  *panel,
                                  GTK_BOX (panel->accounts_vbox),
                                  NULL,
                                  NULL);
+      /*
+       * The above call doesn't set any widgets to visible, so we have to do that.
+       * https://gitlab.gnome.org/GNOME/gnome-online-accounts/issues/56
+       */
+      gtk_widget_show_all (GTK_WIDGET (panel->accounts_vbox));
     }
 
   provider_name = goa_account_get_provider_name (account);
